@@ -59,7 +59,7 @@ enum {
 
 static struct colinfo infos[] = {
 	[COL_PAGES]  = { "PAGES",    1, SCOLS_FL_RIGHT, N_("file data resident in memory in pages")},
-	[COL_RES]    = { "RES",      5, SCOLS_FL_RIGHT, N_("file data resident in memory in bytes")}, 
+	[COL_RES]    = { "RES",      5, SCOLS_FL_RIGHT, N_("file data resident in memory in bytes")},
 	[COL_SIZE]   = { "SIZE",     5, SCOLS_FL_RIGHT, N_("size of the file")},
 	[COL_FILE]   = { "FILE",     4, 0, N_("file name")},
 };
@@ -194,25 +194,21 @@ static int fincore_fd (struct fincore_control *ctl,
 		       off_t *count_incore)
 {
 	size_t window_size = N_PAGES_IN_WINDOW * ctl->pagesize;
-	off_t file_offset;
+	off_t file_offset, len;
 	int rc = 0;
-	int warned_once = 0;
 
-	for (file_offset = 0; file_offset < file_size; file_offset += window_size) {
-		size_t len;
+	for (file_offset = 0; file_offset < file_size; file_offset += len) {
 		void  *window = NULL;
 
 		len = file_size - file_offset;
-		if (len >= window_size)
+		if (len >= (off_t) window_size)
 			len = window_size;
 
-		window = mmap(window, len, PROT_NONE, MAP_PRIVATE, fd, file_offset);
+		/* PROT_NONE is enough for Linux, but qemu-user wants PROT_READ */
+		window = mmap(window, len, PROT_READ, MAP_PRIVATE, fd, file_offset);
 		if (window == MAP_FAILED) {
-			if (!warned_once) {
-				rc = -EINVAL;
-				warn(_("failed to do mmap: %s"), name);
-				warned_once = 1;
-			}
+			rc = -EINVAL;
+			warn(_("failed to do mmap: %s"), name);
 			break;
 		}
 
@@ -311,7 +307,7 @@ int main(int argc, char ** argv)
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	while ((c = getopt_long (argc, argv, "bno:JrVh", longopts, NULL)) != -1) {
 		switch (c) {
@@ -331,8 +327,7 @@ int main(int argc, char ** argv)
 			ctl.raw = 1;
 			break;
 		case 'V':
-			printf(UTIL_LINUX_VERSION);
-			return EXIT_SUCCESS;
+			print_version(EXIT_SUCCESS);
 		case 'h':
 			usage();
 		default:
@@ -369,9 +364,29 @@ int main(int argc, char ** argv)
 
 	for (i = 0; i < ncolumns; i++) {
 		const struct colinfo *col = get_column_info(i);
+		struct libscols_column *cl;
 
-		if (!scols_table_new_column(ctl.tb, col->name, col->whint, col->flags))
+		cl = scols_table_new_column(ctl.tb, col->name, col->whint, col->flags);
+		if (!cl)
 			err(EXIT_FAILURE, _("failed to allocate output column"));
+
+		if (ctl.json) {
+			int id = get_column_id(i);
+
+			switch (id) {
+			case COL_FILE:
+				scols_column_set_json_type(cl, SCOLS_JSON_STRING);
+				break;
+			case COL_SIZE:
+			case COL_RES:
+				if (!ctl.bytes)
+					break;
+				/* fallthrough */
+			default:
+				scols_column_set_json_type(cl, SCOLS_JSON_NUMBER);
+				break;
+			}
+		}
 	}
 
 	for(; optind < argc; optind++) {
